
# The following code implements a posum method using an m=3 t.p.r.s.
# or as a gam with R,f 1-d smooths and death rate an m=2 t.p.r.s.

# first simulation testing code....

pcdr<-function(a,t)
# Per capita death rate for simulation at times t ages a (arrays)
{ mu<-dnorm(a,mean=15,sd=10)*dnorm(t,mean=30,sd=20)*100+0.005
}

rec<-function(t)
# recruitment into population at time t
{ dnorm(t,mean=30,sd=20)*1000
}


survival<-function(a,t)
# survival from birth to age a time t, obtained from pcdr 
{ dt<-0.1 # default timestep
  n<-round(a/dt)+2 # number of points at which to evaluate
  dt<-a/(n-1)
  av<-(0:(n-1))*dt;tv<-av+t-a
  mu<-pcdr(av,tv);inter<-sum(mu)-0.5*mu[1]-0.5*mu[n];inter<-inter*dt
  sur<-exp(-inter)    
}

population<-function(a,t)
# population per unit age interval at age a time t, obtained from rec() and pcdr()
{ rec(t-a)*survival(a,t)
}

population.surface<-function(a1,t1,m,con=FALSE)
# produce population surface on m by m grid....
{ pop<-matrix(0,m,m)
  da<-a1/(m-1);dt<-t1/(m-1)
  t<-a<-0:(m-1);a<-a*da;t<-t*dt
  for (i in 1:m) for (j in 1:m) pop[i,j]<-population(a[i],t[j])
  if (con) contour(a,t,pop) 
  else persp(a,t,pop,theta=60,col="white")
  a<-list(a=a,t=t,pop=pop)  
}

death.surface<-function(a1,t1,m,con=FALSE)
# produce death surface on m by m grid....
{ death<-matrix(0,m,m)
  da<-a1/(m-1);dt<-t1/(m-1)
  t<-a<-0:(m-1);a<-a*da;t<-t*dt
  for (i in 1:m) for (j in 1:m) death[i,j]<-population(a[i],t[j])*pcdr(a[i],t[j])
  if (con) contour(a,t,death) 
  else persp(a,t,death,theta=60,col="white")
  a<-list(a=a,t=t,death=death)  
}


sim.age.bound<-function(stage,t)
# simulation stage boundaries
{ a<-c(0.0,1.0,3.0,6.0,7.5,10.0,12.0,16.0,19.0,21.0,25.0,30.0,40.0)
  mult<-1+0.1*(1-cos(t/20))
  a[stage]*mult 
}


population.data<-function(t=(0:20)*5,ns=11,sc=1,fam="n",adult=FALSE)
# simulate some population data at sample times t, with age boundaries given in 
# a0 and a1. 
# fam can be "n", "p" or "g" for normal, poisson or gamma errors - sc is the scale 
# parameter
{ m<-10 # number of integration points
  nt<-length(t)  # number of sample points
  stage<-y<-ts<-lo<-hi<-0 
  if (adult) ns<-ns+1
  for (i in 1:nt) for (j in 1:ns)
  { a1<-sim.age.bound(j+1,t[i])
    a0<-sim.age.bound(j,t[i])
    da<-(a1-a0)/(m-1)
    ind<-(i-1)*ns+j
    y[ind]<-0.5*population(a0,t[i])
    for (k in 2:(m-1)) y[ind]<-y[ind]+population(a0+(k-1)*da,t[i])
    y[ind]<-y[ind]+0.5*population(a1,t[i])
    y[ind]<-y[ind]*da
    lo[ind]<-a0;hi[ind]<-a1;ts[ind]<-t[i];stage[ind]<-j
  }
  if(adult) hi[stage==ns]<-NA
  if (fam=="n")
  { y<-y+rnorm(length(y))*sc
    y<-pmax(0,y)
  } else
  if (fam=="p")
  y<-rpois(rep(1,length(y)),y)
  else if (fam=="g")
  y<-rgamma(rep(1,length(y)),y/sc,sc)
  list(y=y,a0=lo,a1=hi,t=ts,stage=stage)
}

# now fitting code.....

spl.coef.conv<-function(z)
# takes an object z containing equally lengthed arrays
# z$x, z$y, z$b, z$c, z$d defining a cubic spline interpolating 
# z$x, z$y and forces z$c and z$d to be consistent with z$y and
# z$b (gradient of spline). This is intended for use in conjunction
# with Hyman's monotonicity filter.
# Note that R's spline routine has s''(x)/2 as c and s'''(x)/6 as d.
{ n<-length(z$x)
  h<-z$x[2:n]-z$x[1:(n-1)]
  y0<-z$y[1:(n-1)];y1<-z$y[2:n]
  b0<-z$b[1:(n-1)];b1<-z$b[2:n]
  cc<- -(3*(y0-y1)+(2*b0+b1)*h)/h^2 
  c1<- (3*(y0[n-1]-y1[n-1])+(b0[n-1]+2*b1[n-1])*h[n-1])/h[n-1]^2
  dd<- (2*(y0-y1)/h+b0+b1)/h^2
  d1<-dd[n-1]
  z$c<-c(cc,c1);z$d<-c(dd,d1)
  z 
}


hyman.filter<-function(z) 
# Filters cubic spline function to yield co-monotonicity in accordance with
# Hyman (1983) SIAM J. Sci. Stat. Comput. 4(4):645-654, z$x is knot position
# z$y is value at knot z$b is gradient at knot. See also Dougherty, Edelman and
# Hyman 1989 Mathematics of Computation 52: 471-494. 
{ n<-length(z$x)
  ss<-(z$y[2:n]-z$y[1:(n-1)])/(z$x[2:n]-z$x[1:(n-1)])
  S0<-c(ss[1],ss)
  S1<-c(ss,ss[n-1])
  sig<-z$b
  ind<-(S0*S1>0)
  sig[ind]<-S1[ind]
  ind<-(sig>=0)
  if (sum(ind)) z$b[ind]<-pmin(pmax(0,z$b[ind]),3*pmin(abs(S0[ind]),abs(S1[ind])))
  ind<-!ind
  if (sum(ind)) z$b[ind]<-pmax(pmin(0,z$b[ind]),-3*pmin(abs(S0[ind]),abs(S1[ind])))
  z
}

cm.splinefun<-function(x, y = NULL, method = "fmm",gulim=0) 
# modification of base package splinefun to produce co-monotonic interpolant
# by Hyman Filtering. if gulim!=0 then it is taken as the upper limit on the
# gradient. 
{   x <- xy.coords(x, y)
    y <- x$y
    x <- x$x
    n <- length(x)
    method <- match(method, c("periodic", "natural", "fmm"))
    if (is.na(method)) 
        stop("splinefun: invalid interpolation method")
    if (any(diff(x) < 0)) {
        z <- order(x)
        x <- x[z]
        y <- y[z]
    }
    if (method == 1 && y[1] != y[n]) {
        warning("first and last y values differ in spline - using y[1] for both")
        y[n] <- y[1]
    }
    z <- .C("spline_coef", method = as.integer(method), n = n, 
        x = as.double(x), y = as.double(y), b = double(n), c = double(n), d = double(n), 
        e = double(if (method == 1) n else 0), PACKAGE = "base")
    
    z$y<-z$y-z$x*gulim # trick to impose upper
    z$b<-z$b-gulim     # limit on interpolator gradient
    
    z<-hyman.filter(z) # filter gradients for co-monotonicity
    
    z$y<-z$y+z$x*gulim # undo trick 
    z$b<-z$b+gulim     # transformation
    
    z<-spl.coef.conv(z) # force other coefficients to consistency
   
    rm(x, y, n, method)
    function(x) {
        .C("spline_eval", z$method, length(x), x = as.double(x), 
            y = double(length(x)), z$n, z$x, z$y, z$b, z$c, z$d, 
            PACKAGE = "base")$y
    }
}


posum.atplot<-function(data,n.grid=100)
# plot illustrating the stage boundary ranges assumed for data,
# along with sample times
{ 
  stages<-unique(data$stage) 
  ns<-max(stages)
  # find the plot range
  alim<-c(min(data$a0),max(data$a0,max(data$a1[!is.na(data$a1)])))
  tlim<-range(data$t)
  adults<-FALSE
  if (sum(is.na(data$a1[data$stage==ns]))>0) adults<-TRUE
  if (adults&&sum(!is.na(data$a1[data$stages==ns]))>0)
  stop("Final stage must be all adult or all sub-adult!")
  if (adults) 
  { a.len<-(alim[2]-alim[1])/(2*ns) # length of line segments to plot for adults
    alim[2]<-alim[2]+a.len 
  }
  plot(tlim,alim,type="n",xlab="time",ylab="age",main="Stage boundary plot")
  t<-seq(from=tlim[1],to=tlim[2],length=n.grid)
  for (i in 1:length(stages)) 
  { ind<-data$stage==stages[i]
    if (sum(ind)<3) stop("Insufficient age range data for one stage")
    tk<-data$t[ind];ak<-data$a0[ind]
    sf<-cm.splinefun(tk,ak, method = "fmm",gulim=1) 
    lines(t,sf(t))
  }
  if (!adults)
  { ind<-data$stage==stages[ns]
    tk<-data$t[ind];ak<-data$a1[ind]
    sf<-cm.splinefun(tk,ak, method = "fmm",gulim=1) 
    lines(t,sf(t))
  }
  # plot the age limit only information
  ind<-is.na(data$y)
  if (sum(ind)) points(data$t[ind],data$a0[ind],pch=20)
  ind2<-ind&!is.na(data$a1)
  if (sum(ind2)) points(data$t[ind2],data$a1[ind2],pch=20)
  
  # plot range covered by each other datum (avoiding loops!)
  ind<-!is.na(data$y) # data for which there are population data
  a0<-data$a0[ind];a1<-data$a1[ind];tt<-data$t[ind]
  ind2<-is.na(a1) # find adult records
  if (sum(ind2)) a1[ind2]<-a0[ind2]+a.len # setting adult line lengths
  n<-length(a0)     # number of line segments to plot
  t<-array(NA,3*n);a<-array(NA,3*n)
  ind<-1:n*3-2      
  t[ind]<-tt;a[ind]<-a0
  ind<-ind+1
  t[ind]<-tt;a[ind]<-a1
  # t and a now contain age ranges for each y datum, separated by NA's
  lines(t,a,col=2) 
  # now deal with point estimates of surface height
  ind<-a0==a1
  if (sum(ind)) points(tt[ind],a0[ind],pch=20,col=2)
}



posum.con<-function(b,amax,data,fdi=0.01,m=40,adults=FALSE)
# obtain constraint matrix ensuring +ve death rates and population 
# C%*%b$coeff >= 0 approximately ensures +ve pop and death rates
# amax(t) is a function returning the upper boundary on the population 
# surface at time t (t can be a vector in which case amax is vector 
# valued)
{ a0<-min(data$a0);
  a1<-max(amax(unique(data$t)))
  t0<-min(data$t);t1<-max(data$t)
  ts<-seq(t0,t1,length=m);dt<-ts[2]-ts[1]
  as<-seq(a0,a1,length=m);da<-as[2]-as[1]
  fd<-min(c(da,dt))*fdi
  tt<-rep(ts,rep(m,m));aa<-rep(as,m)
  t<-tt[aa<=amax(tt)];a<-aa[aa<=amax(tt)] # getting rid of parts of mesh above upper age boundary
  rm(aa,tt)
  n<-length(t)
  if (adults) # need "by" variables
  d<-data.frame(t=t,a=a,adult=rep(0,n),juvenile=rep(1,n))
  else d<-data.frame(t=t,a=a)
#  xp<-predict.gam(b,d,type="lpmatrix")
#  xp<-posum.pop.matrix(b,d)
#  t<-t+fd;a<-a+fd
#  d$t<-t;d$a<-a
  #xd<-predict.gam(b,d,type="lpmatrix")
  #xd<-(xp-xd)/fd # finite difference gradients
  xd<-posum.death.matrix(b,d,fd)
  # positive death rates ensured, but also need +ve population!
  as<-amax(ts) # surface upper age limit array
  # surface needs to be non-negative at upper age and time boundaries...
  a<-c(as,seq(a0,as[m],length=m));t<-c(ts,rep(t1,m))
  if (adults) d<-data.frame(t=t,a=a,adult=rep(0,2*m),juvenile=rep(1,2*m))
  else d<-data.frame(t=t,a=a)
  #xp<-predict.gam(b,d,type="lpmatrix")
  xp<-posum.pop.matrix(b,d)
  # now have to deal with adult related constraints if adult stage is present.....
  if (adults)
  { # get approximate damax/dt
    da.dt<-(amax(ts+0.5*fd)-amax(ts-0.5*fd))/fd
    da.dt<-1-pmin(da.dt,1)
    # form first data frame for death condition
    d<-data.frame(t=ts-0.5*fd,a=as,juvenile=da.dt/2,adult=rep(1/fd,m))
    #ad<-predict.gam(b,d,type="lpmatrix")
    ad<-posum.pop.matrix(b,d)
    # second data frame....
    d<-data.frame(t=ts+0.5*fd,a=as,juvenile=da.dt/2,adult=rep(-1/fd,m))
    #ap<-predict.gam(b,d,type="lpmatrix")
    ap<-posum.pop.matrix(b,d)
    ad<-ad+ap # finite difference approximation to pos death condition
    # and finally the positive adult population condition
    d<-data.frame(t=ts,a=as,juvenile=rep(0,m),adult=rep(1,m))
    #ap<-predict.gam(b,d,type="lpmatrix")    
    ap<-posum.pop.matrix(b,d)
    # now put everything into one constraint matrix
    c.rows<-dim(xp)[1]+dim(xd)[1]+dim(ad)[1]+dim(ap)[1]
    C<-matrix(0,c.rows,dim(xp)[2])
    k0<-1;k1<-dim(xd)[1];C[k0:k1,]<-xd
    k0<-k1+1;k1<-k1+dim(xp)[1];C[k0:k1,]<-xp
    k0<-k1+1;k1<-k1+dim(ad)[1];C[k0:k1,]<-ad
    k0<-k1+1;k1<-k1+dim(ap)[1];C[k0:k1,]<-ap     
  } else
  { C<-matrix(0,dim(xp)[1]+dim(xd)[1],dim(xp)[2])
    C[1:dim(xd)[1],]<-xd
    C[(1+dim(xd)[1]):dim(C)[1],]<-xp
  }
  C 
}

age.max<-function(data)
# Returns a function (amax(t)) giving the upper age boundary of the 
# population surface at t, given a posum input data frame/list
{ # first find the final stage
  ms<-max(data$stage)
  ind<-data$stage==ms
  if (sum(is.na(data$a1[ind]))) # then there are adults
  { a<-data$a0[ind];t<-data$t[ind]
    if (length(a)<3) stop("Insufficient adult lower age limit data")
  } else # all juveniles
  { a<-data$a1[ind];t<-data$t[ind]
    if (length(a)<3) stop("Insufficient upper age limit data for final stage")
  }
  # check for sensible limit data
  n<-length(a)
  a0<-a[1:(n-1)];a1<-a[2:n]
  t0<-t[1:(n-1)];t1<-t[2:n]
  if (sum((a1-a0)>(t1-t0))) # then boundary increases impossibly fast
  stop("Age limit data implies age at upper boundary of population surface\\ increasing at more than 1 day per day!")
  rm(a0,a1,t0,t1,ind,ms)
  # return spline function through available age limits, filtered to have maximum 
  # gradient 1 
  cm.splinefun(t,a,gulim=1) 
}

stage.bound<-function(x,stage,low=TRUE)
# obtains stage boundary of stage from object x, which is either a posum 
# input data list or a posum fitted model object
{ if (!is.null(x$data)) d<-x$data else d<-x # d now contains the data object
  ind<-d$stage==stage
  if (sum(ind)==0) stop("Stage non-existent in data")
  t<-d$t[ind];
  if (low) a<-d$a0[ind] else a<-d$a1[ind]
  if (sum(is.na(a))) stop ("NA's in age boundary data")
  if (sum(is.na(t))) stop ("NA's in time data")
  # check for sensible limit data
  n<-length(a)
  a0<-a[1:(n-1)];a1<-a[2:n]
  t0<-t[1:(n-1)];t1<-t[2:n]
  if (sum((a1-a0)>(t1-t0))) # then boundary increases impossibly fast
  stop("Stage boundary age increasing at more than 1 day per day according to data!")
  rm(a0,a1,t0,t1,ind)
  cm.splinefun(t,a,gulim=1) 
}

posum.stage<-function(x,lo=1,hi=lo,t=seq(min(x$data$t),max(x$data$t),length=100),
                      n.int=x$options$n.int,fdi=x$options$fdi,plot=TRUE,aggregate=FALSE)
# integrates pop and death over stages low to hi inclusive at each of points in t.
# n.int integration points are used per stage (average) and finite difference intervals used 
# for death are fdi times the integration interval. 
{ if (hi<lo) stop("don't be silly")
  if (hi>max(x$data$stage)) stop("upper stage limit above number of available stages")
  if (lo<1) stop("lowest stage must be > 0")
  if (x$adults&&hi==max(x$data$stage)) adults<-TRUE else adults<-FALSE
  n.stages<-hi-lo+1
  n<-length(t)
  if (n.stages==1&&adults) juveniles<-FALSE else juveniles<-TRUE
  if (juveniles)
  { n.int<-n.int*(n.stages-1*adults)
    lo.fn<-stage.bound(x,lo,low=TRUE)
    if (adults) j.hi<-hi-1 else j.hi<-hi
    hi.fn<-stage.bound(x,j.hi,low=FALSE)
    da<-(hi.fn(t)-lo.fn(t))/(n.int-1) # integration mesh at each time point
    fd<-fdi*mean(da) # finite difference interval
    mult<-0:(n.int-1)
    ida<-rep(da,rep(n.int,n))
    ai<-ida*mult+rep(lo.fn(t),rep(n.int,n)) # using recycling rule
    ti<-rep(t,rep(n.int,n))
    int.d<-data.frame(a=ai,t=ti,juvenile=rep(1,n.int*n),adult=rep(0,n.int*n))        
    # pm<-predict.gam(b,int.d,type="lpmatrix") # population integration matrix - old code
    pm<-posum.pop.matrix(b,int.d) # population integration matrix
    ind<-rep(0,n.int);ind[1]<-ind[n.int]<-1;index<-rep(ind,n) # interval end index
    pm[index,]<-pm[index,]*0.5 # trapezoidal rule multipliers at interval ends
    pm<-pm*ida # recycling rule - works col by col multiplying rows by right interval 
    sneak<-c(rep(1,n.int),rep(0,n*n.int))
    imat<-matrix(c(rep(sneak,n-1),rep(1,n.int)),n,n*n.int,byrow=TRUE) # matrix to sum pm appropriately
    jpm<-imat%*%pm
    # now do same, but finite differencing for death 
    pm<-posum.death.matrix(b,int.d,fd)
    pm[index,]<-pm[index,]*0.5 # trapezoidal rule multipliers at interval ends
    pm<-pm*ida # recycling rule - works col by col multiplying rows by right interval 
    jdm<-imat%*%pm
    rm(sneak,imat)
    # old code from here ....
    #int.d$t<-ti-fd/2;int.d$a<-ai-fd/2
    #pm<-predict.gam(b,int.d,type="lpmatrix") # population integration matrix
    #pm[index,]<-pm[index,]*0.5 # trapezoidal rule multipliers at interval ends
    #pm<-pm*ida # recycling rule - works col by col multiplying rows by right interval 
    #back.diff<-imat%*%pm
    #int.d$t<-ti+fd/2;int.d$a<-ai+fd/2
    #pm<-predict.gam(b,int.d,type="lpmatrix") # population integration matrix
    #pm[index,]<-pm[index,]*0.5 # trapezoidal rule multipliers at interval ends
    #pm<-pm*ida # recycling rule - works col by col multiplying rows by right interval 
    #forward.diff<-imat%*%pm
    #jdm<-(back.diff-forward.diff)/fd
    #rm(pm,back.diff,forward.diff,int.d,ai,ti,sneak,imat) # remove larger objects
    # now jpm and jdm contain matrices which will give aggegated population
    # and death rates at each of the times in t, when applied to the parameter 
    # vector. 
  }
  if (adults) # then mapping matrices are required for adults too 
  { if (!juveniles) 
    { fd<-(max(t)-min(t))/length(t);
      fd<-fdi*fd/n.int
    }
    # get approximate damax/dt
    da.dt<-(x$amax.fn(t+0.5*fd)-x$amax.fn(t-0.5*fd))/fd
    da.dt<-1-pmin(da.dt,1)
    amax<-x$amax.fn(t)
    # form first data frame for death fd approx
    d<-data.frame(t=t-0.5*fd,a=amax,juvenile=da.dt/2,adult=rep(1/fd,n))
    ad<-posum.pop.matrix(b,d)
    # second data frame....
    d<-data.frame(t=t+0.5*fd,a=amax,juvenile=da.dt/2,adult=rep(-1/fd,n))
    #ap<-predict.gam(b,d,type="lpmatrix")
    ap<-posum.pop.matrix(b,d)
    ad<-ad+ap # finite difference approximation to death rate 
    # and finally the adult population
    d<-data.frame(t=t,a=amax,juvenile=rep(0,n),adult=rep(1,n))
    #ap<-predict.gam(b,d,type="lpmatrix")    
    ap<-posum.pop.matrix(b,d)
    # now ad and ap are the adult equivalents of jdm and jpm above
  }
  if (adults&&juveniles)
  { mp<-ap+jpm;md<-ad+jdm;rm(ap,jpm,ad,jdm)}
  else 
  { if (adults) {mp<-ap;md<-ad;rm(ap,ad)}
    else {mp<-jpm;md<-jdm;rm(jpm,jdm)}
  }
  if (aggregate) # then integration over range of t is required
  { t0<-t[1:(n-1)];t1<-t[2:n];dt<-t1-t0
    idt<-c(0.5*dt,0)+c(0,0.5*dt)
    mp<-idt%*%mp
    md<-idt%*%md
  } 
  p<-mp%*%x$coefficients;p<-pmax(0,p)
  p.se<-diag(mp%*%x$Vp%*%t(mp))^0.5
  d<-md%*%x$coefficients;d<-pmax(0,d)
  d.se<-diag(md%*%x$Vp%*%t(md))^0.5
  m<-d/p;m[p==0]<-NA
  m.se<-(p.se^2*(d/p^2)^2+d.se^2/p^2-(d/p^3)*diag(mp%*%x$Vp%*%t(md)))^0.5
  if (plot&&!aggregate)
  { op<-par(mfrow=c(3,1))
    ll<-p-2*p.se;ll<-pmax(ll,0)
    ylims<-c(min(ll),max(p+2*p.se))
    plot(t,p,type="l",ylim=ylims,xlab="time",ylab="population")
    lines(t,ll,lty=2,col=2);lines(t,p+2*p.se,lty=2,col=2)
    ll<-d-2*d.se;ll<-pmax(ll,0)
    ylims<-c(min(ll),max(d+2*d.se))
    plot(t,d,type="l",ylim=ylims,xlab="time",ylab="total deaths")
    lines(t,ll,lty=2,col=2);lines(t,d+2*d.se,lty=2,col=2)
    ll<-m-2*m.se;ll<-pmax(ll,0)
    ylims<-c(min(ll),max(m+2*m.se))
    plot(t,m,type="l",ylim=ylims,xlab="time",ylab="p.c.d.r.")
    lines(t,ll,lty=2,col=2);lines(t,m+2*m.se,lty=2,col=2)
    par(op)
  }
  res<-data.frame(p=p,d=d,m=m,p.se=p.se,d.se=d.se,m.se=m.se)
}

posum.pop.matrix<-function(b,d,t0=0,point.wise=TRUE)
# routine for obtaining matrix mapping parameters to vector to evaluated population values
# at ages and times given in dataframe d. b is the (augmented) gam object for the posum model
# If point.wise is true then a predict.gam call is made for each a,t point to save memory
# If juvenile record is null in d and model.type==2 then it is assumed that the juvenile 
# by variable should be 1 and the adult one 0.
{ if (b$model.type==1) # population is the smooth function
  { ux<-predict.gam(b,d,type="lpmatrix")}
  else if (b$model.type==2) # death is the smooth function
  { dd<-d
    if (is.null(dd$juvenile)) # if juvenile record missing altogether assume all juvenile
    { dd$juvenile<-dd$a*0+1;dd$adult<-dd$juvenile*0 }
    dd$rec<-dd$f0<-dd$juvenile
    m<-round(dd$a/b$target.dt);
    index<-dd$t-dd$a>t0
    m[index]<-round((dd$t[index]-t0)/b$target.dt)
    m<-m+1
    m[m<2]<-2 # number of points to use in evaluation of death rate integral
    m[d$juvenile==0]<-0 # no juvenile information required  
    n<-length(d$a)+sum(m) # number of rows in expanded data.frame
    if (point.wise) # create mapping matrix in advance 
    { ux<-matrix(0,length(d$a),length(b$coefficients))
    } else # create one big data frame and summation matrix
    { ed<-data.frame(a=1:n*0,t=1:n*0,rec=1:n*0,f0=1:n*0,adult=1:n*0,juvenile=1:n*0)
      k<-1
      summer<-matrix(0,length(d$a),n)
    }
    for (i in 1:length(d$a)) # loop through a,t values, creating expanded data.frame  
    { if (point.wise) 
      { summer<-matrix(0,1,1+m[i]) 
        k<-m[i]+1
        ed<-data.frame(a=1:k*0,t=1:k*0,rec=1:k*0,f0=1:k*0,adult=1:k*0,juvenile=1:k*0)
        k<-1
      } 
      summer[1,k:(k+m[i])]<-1 # filling in relevant parts of summation matrix
      if (index[i]) # recruitment is from R(t)
      { if (m[i]>0) # juvenile information required 
        { ed$rec[k]<-dd$juvenile[i];ed$t[k]<-dd$t[i]-dd$a[i] # recruitment R(t-a) row
          ed$adult[k]<-dd$adult[i]
          k<-k+1;
          dt<-dd$a[i]/(m[i]-1)
          juv<-rep(1,m[i]);juv[1]<-juv[m[i]]<-0.5;juv <- -juv*dt*dd$juvenile[i]
          ed$juvenile[k:(k+m[i]-1)]<-juv # the death integral rows....
          ed$a[k:(k+m[i]-1)]<-0:(m[i]-1)*dt
          ed$t[k:(k+m[i]-1)]<-dd$t[i]-dd$a[i]+0:(m[i]-1)*dt
          k<-k+m[i]
        } else 
        { ed$adult[k]<-dd$adult[i] # adult only evaluation
          ed$t[k]<-dd$t[i]
        }
      } else # recruitment is from f(a) 
      { if (m[i]>0) # juvenile info required
        { ed$f0[k]<-dd$juvenile[i];ed$a[k]<-dd$a[i]-dd$t[i]+t0
          ed$adult[k]<-dd$adult[i]
          k<-k+1
          dt<-(dd$t[i]-t0)/(m[i]-1)
          juv<-rep(1,m[i]);juv[1]<-juv[m[i]]<-0.5;juv <- -juv*dt*dd$juvenile[i]
          ed$juvenile[k:(k+m[i]-1)]<-juv # the death integral rows....
          ed$a[k:(k+m[i]-1)]<-dd$a[i]-dd$t[i]+t0+0:(m[i]-1)*dt
          ed$t[k:(k+m[i]-1)]<-t0+0:(m[i]-1)*dt
          k<-k+m[i]
        } else  
        { ed$adult[k]<-dd$adult[i] # adult only evaluation
          ed$t[k]<-dd$t[i]
        }
      }
      if (point.wise)
      { er<-predict.gam(b,ed,type="lpmatrix") 
        ux[i,]<-summer%*%er
        rm(ed,er,summer)
      }
    }
    if (!point.wise) # just a single call
    { ux<-predict.gam(b,ed,type="lpmatrix")
      ux<-summer%*%ux
    }
  } else
  { ux<-predict.gam(b,d,type="lpmatrix")
  }
  ux<-ux
}

posum.death.matrix<-function(b,d,fd)
# routine for obtaining matrix mapping parameters to vector of evaluated death surface values
# at ages and times given in dataframe d. b is the (augmented) gam object for the posum model
# fd is the finite difference interval to use for this when finite differencing. 
# This routine does not evaluate adult death rates.
{ if (b$model.type==1) # then population surface is the smooth function
  { df<-d
    df$t<-df$t-fd/2;df$a<-df$a-fd/2
    ux<-predict.gam(b,df,type="lpmatrix")
    df$t<-df$t+fd;df$a<-df$a+fd
    ux<-(ux-predict.gam(b,df,type="lpmatrix"))/fd
  } else
  if (b$model.type==2) # total death rate is the smooth surface
  { dd<-d
    dd$adult<-dd$f0<-dd$rec<-dd$t*0;dd$juvenile<-dd$t*0+1 # prepare "by" variables
    ux<-predict.gam(b,dd,type="lpmatrix")
  }
  ux<-ux
}


posum.X<-function(data,b,ni=10)
# get design matrix for posum model, given a gam object specifying an appropriate basis
# the data object and ni - the steps per stage to use for integration
{ n<-length(data$y)
  # make data frame to use for integration.....
  nia<-rep(ni-1,n);        # array of number of integration intervals for each y
  nia[data$a0==data$a1]<-1 # point estimates only require one evaluation
  ad.ind<-is.na(data$a1)   # adult indexing array
  adults<-as.logical(sum(ad.ind))
  nia[ad.ind]<-1           # adults only require 1 evaluation
  dt<-(data$a1-data$a0)/nia;nia[nia!=1]<-nia[nia!=1]+1
  dt[ad.ind]<-0
  t<-rep(data$t[1],nia[1])
  a<-data$a0[1]+(1:nia[1])*dt[1]-dt[1]
  if (adults) # need by variables
  { if (ad.ind[1]) { adult<-1;juvenile<-0} 
    else {adult<-rep(0,nia[1]);juvenile<-rep(1,nia[1])}
  }
  for (i in 2:n)
  { t<-c(t,rep(data$t[i],nia[i]))
    a<-c(a,data$a0[i]+(1:nia[i])*dt[i]-dt[i])
    if (adults) # need by variables too
    { if (ad.ind[i]) 
      { adult<-c(adult,rep(1,nia[i]));juvenile<-c(juvenile,rep(0,nia[i]))} 
      else 
      { adult<-c(adult,rep(0,nia[i]));juvenile<-c(juvenile,rep(1,nia[i]))}
    }
  }
  if (adults) d<-data.frame(t=t,a=a,adult=adult,juvenile=juvenile)
  else d<-data.frame(t=t,a=a,juvenile=a*0+1,adult=0*a)
  # call predict.gam to get matrix mapping parameters to surface at large set of points...
  # ux<-predict.gam(b,d,type="lpmatrix") # raw matrix for creating X - old code
  ux<-posum.pop.matrix(b,d,data$t[1]) # raw matrix for creating X
  X<-matrix(0,n,length(b$coeff));k<-0
  # following could be improved - see posum.stage()
  for (i in 1:n)  # use ux and Simpsons rule to get X matrix
  { X[i,]<-0.5*ux[1+k,]
    if (nia[i]>2) for (j in 2:(nia[i]-1)) X[i,]<-X[i,]+ux[j+k,]
    X[i,]<-X[i,]+0.5*ux[nia[i]+k,]
    if (nia[i]>1) X[i,]<-X[i,]*dt[i]
    k<-k+nia[i]
  }
  X
}

posum.check<-function(data)
# do a health check on the supplied data and return TRUE/FALSE depending 
# on presence/absence of adult data
{ if (sum(is.na(data$a0))) stop("NA's in lower age boundary data")
  if (sum(is.na(data$stage))) stop("NA's in stage index data")
  ms<-max(data$stage)
  if (sum(is.na(data$a1[data$stage!=ms])))
  stop("NA's in pre-adult upper stage boundary data")
  n.na<-sum(is.na(data$a1[data$stage==ms]))
  if(n.na!=0&&n.na!=length(data$a1[data$stage==ms])) 
  stop("Mixture of data and NA in final stage upper age limits")
  if (sum(is.na(data$t))) stop("NA's in time data")
  if (sum(is.na(data$a1[data$stage==ms]))) ret<-TRUE else ret<-FALSE
  ret
}

posum.options<-function(con.mesh=40,fdi=0.01,n.int=10,m.dint=3)
# user controllable options function for posum.
# m.dint is used to control the integration timestep used to integrate the 
# death surface. n.int*(number of stages)*m.dint steps are used to the 
# maximum age.
{ list(con.mesh=con.mesh,fdi=fdi,n.int=n.int,m.dint=m.dint)
}


posum<-function(data,constrain=TRUE,fam="n",jk=50,ak=10,fk=5,rk=10,
                options=posum.options(),control=gam.control(),model.type=1)
# routine for fitting population surface model to data 
# data$y - pop data; data$t - sampling times; data$a0 - lower are bounds
# data$a1 - upper age bounds NA for adults; data$stage - life history stage 
# fam can be "n", "p", or "g" for normal, poisson or Gamma families 
# link is always identity, of course. 
{ ni<-options$n.int #integration steps
  adults<-posum.check(data) # check the data for obvious problems and presence of adults
  full.data<-data
  # now drop missing data....
  ind<-!is.na(data$y)
  dy<-data$y[ind]
  dt<-data$t[ind]
  dstage<-data$stage[ind]
  da0<-data$a0[ind]
  da1<-data$a1[ind]
  data<-list(y=dy,t=dt,stage=dstage,a0=da0,a1=da1)
  rm(dy,dt,dstage,da0,da1)
  # .... done
  # create "by" variables for adults, juveniles 
  if (adults)
  { maxs<-max(data$stage)
    adult.by<-as.numeric(is.na(data$a1))
    juvenile.by<-1-adult.by
  }
  n<-length(data$y)
  ind<-!is.na(data$a1)
  x<-matrix(0,n+sum(ind),2)
  x[,1]<-c(data$t,data$t[ind]);x[,2]<-c(data$a0,data$a1[ind])
  uniquecombs(x)->x # find all the unique a,t combinations in the data
  t<-x[,1];a<-x[,2];y<-t+a+rnorm(length(a)) # fake some data at these locations
  if (adults) # data faking needs to deal with adults
  { ad.t<-unique(t) # times to use for basis construction
    ad.a<-ad.t*0    # ages to use (irrelavant)
    na<-length(ad.t);nj<-length(a)
    ad.y<-ad.t+rnorm(na)
    juvenile<-c(rep(1,nj),rep(0,na)) # by variable for juvenile part 
    adult<-c(rep(0,nj),rep(1,na)) # by variable for juvenile part 
    t<-c(t,ad.t);a<-c(a,ad.a);y<-c(y,ad.y) # augmented data
  }
  if (model.type==1) # then smooth surface is population
  { if (adults)
    G<-gam.setup(y~juvenile+adult-1+s(a,t,m=3,k=jk,by=juvenile)+s(t,m=2,k=ak,by=adult))
    else G<-gam.setup(y~s(a,t,m=3,k=jk))
    G<-GAMsetup(G) # basis for model setup
    if (adults) G$min.edf<-null.space.dimension(m=3,d=2)+null.space.dimension(m=2,d=1)
    else G$min.edf<-null.space.dimension(m=3,d=2)
    # need a design matrix for the model
    if (adults)
    b<-gam(y~juvenile+adult-1+s(a,t,m=3,k=jk,by=juvenile)+s(t,m=2,k=ak,by=adult))
    else b<-gam(y~s(a,t,m=3,k=jk)) # get a gam object (wrong coefficients)
    b$model.type<-1 
  } else
  if (model.type==2) # total death is the smooth surface
  { 
    if (adults)
    { rec<-f0<-juvenile # rec and f0 apply to juvenile part of model
      G<-gam.setup(y~juvenile+adult+rec+f0-1+s(a,t,m=2,k=jk,by=juvenile)+s(t,m=2,k=ak,by=adult)+
                                           s(a,m=2,k=fk,by=f0)+s(t,m=2,k=rk,by=rec))
    }
    else 
    { rec<-f0<-juvenile<-1+y*0
      G<-gam.setup(y~juvenile+rec+f0-1+s(a,t,m=2,k=jk,by=juvenile)+s(a,m=2,k=fk,by=f0)+
                                           s(t,m=2,k=rk,by=rec))
    }
    # set by variables to something that will make model identifiable in gam call .... 
    rec<-runif(length(y));f0<-runif(length(y))
    G<-GAMsetup(G) # basis for model setup
    if (adults) G$min.edf<-null.space.dimension(m=2,d=2)+3*null.space.dimension(m=2,d=1)
    else G$min.edf<-null.space.dimension(m=2,d=2)+2*null.space.dimension(m=2,d=1)
    # need a design matrix for the model....
    if (adults)
    b<-gam(y~juvenile+adult+rec+f0-1+s(a,t,m=2,k=jk,by=juvenile)+s(t,m=2,k=ak,by=adult)+
                                           s(a,m=2,k=fk,by=f0)+s(t,m=2,k=rk,by=rec))
    else b<-gam(y~juvenile+rec+f0-1+s(a,t,m=2,k=jk,by=juvenile)+s(a,m=2,k=fk,by=f0)+
                                           s(t,m=2,k=rk,by=rec)) # get a gam object (wrong coefficients)
    b$model.type<-2 
  }
  amax.fn<-age.max(full.data) # get upper age boundary of surface (using all age data supplied)  
  max.age<-max(amax.fn(full.data$t))
  b$target.dt<-max.age/(options$n.int*options$m.dint*max(data$stage)) # target death integration timestep
  # Now b contains a gam object which can be passed to predict.gam....
  # this means that the integration required for the design matrix formation
  # is easy as are the evaluations required in order to impose constraints
  # print(G$off)
  G$X<-posum.X(data,b,options$n.int) # replace initial design matrix by correct one 
  # need to add constraint tying R(t_0) to f(0) to G$C
  if (model.type==2)
  { C<-matrix(0,dim(G$C)[1]+1,dim(G$C)[2])
    t0<-min(data$t)
    C[1:dim(G$C)[1],]<-G$C
    C[dim(G$C)[1]+1,]<-predict.gam(b,data.frame(t=t0,a=0,juvenile=0,adult=0,rec=1,f0=-1),type="lpmatrix")[1,]
    G$C<-C
  }
  if (constrain)
  { G$Ain<-posum.con(b,amax.fn,data,adults=adults,fdi=options$fdi,m=options$con.mesh) 
    G$bin<-array(0,dim(G$Ain)[1])
    G$p<-array(0,dim(G$Ain)[2])
    if (model.type==1)   
    { G$p[1]<-mean(data$y); 
      if (adults) a.pos<-G$nsdf+G$df[1]-G$min.edf+4
      else a.pos<-G$nsdf+G$df[1]-G$min.edf+2
      G$p[a.pos]<- -G$p[1]/(2*max(a))
      if (adults)
      G$p[2]<-G$p[1]
    } else
    if (model.type==2)
    { G$p[2]<-G$p[3]<-mean(data$y)
      G$p[1]<-G$p[2]/(2*max(a))
      if (adults) G$p[4]<-G$p[3]
    }
    viol<-min(G$Ain%*%G$p); #cat(viol)
    if (viol==0) warning("Initial parameters on constraints in posum()")
    else if (viol<0) 
    { cat("violation = ",viol,"\n")
      stop("Initial parameters violate constraints in posum()")
    }
  }
  # following code modifies G for new problem
  G$y<-data$y
  b.n<-G$n
  G$n<-n
  scale<- -1
  weights<-NULL
  if (is.null(G$offset)) 
        G$offset <- rep(0, G$n)
  if (is.null(weights)) 
        G$w <- rep(1, G$n)
  else G$w <- weights
  if (scale == 0) {
      if (family$family == "binomial" || family$family == "poisson") 
            scale <- 1
        else scale <- -1
  }
  G$sig2 <- scale
  G$sp <- array(-1, G$m)
  G$conv.tol <- control$mgcv.tol
  G$max.half <- control$mgcv.half
  # Now fit the model......
  if (fam=="n") family<-gaussian() else
  if (fam=="p") family<-poisson(link="identity") else
  if (fam=="g") family<-Gamma(link="identity") else
  stop("Family not supported by posum()")

  if (constrain)
  posum.fit(G,family=family,control=control)->object 
  else gam.fit(G)->object
  # following lifted from gam()
  object$covariate.shift <- G$covariate.shift
  names(object$covariate.shift) <- G$vnames[(dim(G$x)[1] - 
        length(G$covariate.shift) + 1):dim(G$x)[1]]
  if (scale < 0) 
        object$gcv.used <- TRUE
  else object$gcv.used <- FALSE
  object$xp <- G$xp
  object$UZ <- G$UZ
  object$Xu <- G$Xu
  object$xu.length <- G$xu.length
  if (G$m > 0) 
  for (i in 1:G$m) if (G$fix[i])  object$edf[i] <- G$df[i] - 1
  term.names <- array("", length(object$coefficients))
  if (G$nsdf > 0) for (i in 1:G$nsdf) term.names[i] <- G$names[i]
  i <- G$nsdf + 1
  if (G$m > 0) 
  for (k in 1:G$m) for (j in 1:G$df[k]) {
            term.names[i] <- paste(G$names[G$nsdf + k], ".", 
                as.character(j), sep = "")
            i <- i + 1
        }
  names(object$coefficients) <- term.names
  object$formula<-object$full.formula <- G$full.formula
  environment(object$full.formula) <- environment(formula) # not sure of this
  object$x <- G$x
  object$by <- G$by
  object$by.exists <- G$by.exists
  object$s.type <- G$s.type
  object$p.order <- G$p.order
  object$dim <- G$dim
  object$call <- b$call
  object$data<-full.data
  object$S<-G$S
  object$model.type<-model.type
  object$amax.fn<-amax.fn
  object$target.dt<-b$target.dt
  object$adults<-adults
  object$options<-options
  rownames(object$x) <- G$vnames
  names(object$x) <- NULL
  class(object) <- "gam"
  object
}


posum.surface<-function(b,fdi=b$options$fdi,m=b$options$con.mesh,type="p",plot=TRUE,con=FALSE,theta=60)
# this routine produces population or death surface data on the specified mesh 
# from the gam object b produced by posum() plots are on an m by m mesh fdi is 
# the proportion of the mesh spacing to use as a finite difference interval for 
# death rate calculations. type="p", "d" or "m" for population, total death rate 
# or per capita death rate respectively. 
{ data<-b$data # the original data
  a0<-min(data$a0);
  a1<-max(b$amax.fn(data$t))
  t0<-min(data$t);t1<-max(data$t)
  da<-(a1-a0)/(m-1);dt<-(t1-t0)/(m-1)
  ts<-as<-0:(m-1);as<-as*da+a0;ts<-ts*dt+t0
  fd<-min(c(da,dt))*fdi
  t<-rep(ts,rep(m,m));a<-rep(as,m)
  d<-data.frame(t=t,a=a,adult=rep(0,m*m),juvenile=rep(1,m*m))
  if (type=="m"||type=="d") # then get death rate information
  { xd<-posum.death.matrix(b,d,fd)
  }
  if (type!="d") # get population information
  { xp<-posum.pop.matrix(b,d)
    xx<-xp%*%b$coeff;rm(xp)
    xx[a>b$amax.fn(t)]<-NA # hide off surface stuff
    zp<-matrix(xx,m,m)
  }
  if (type=="m"||type=="d") # then get death rate information
  { xx<-xd%*%b$coeff;rm(xd)
    xx[a>b$amax.fn(t)]<-NA # hide off surface stuff
    zd<-matrix(xx,m,m)
  }
  if (type=="d")
  { res<-list(a=as,t=ts,z=zd)}
  else
  if (type=="m")
  { zm<-zp
    zm[zm<=0]<-NA
    zm[!is.na(zm)]<-zd[!is.na(zm)]/zp[!is.na(zp)]
    res<-list(a=as,t=ts,z=zm)
  } else
  res<-list(a=as,t=ts,z=zp) # defaults to type=="p"
  if (plot)
  { if (con) contour(as,ts,res$z)
    else persp(as,ts,res$z,theta=theta,xlab="age",ylab="time",zlab="population",col="lightblue",
          zlim=c(0,max(res$z,na.rm=TRUE)))
  }
  res<-res
}


posum.plot<-function(b,fdi=b$options$fdi,m=b$options$con.mesh,con=FALSE,theta=60)
# this routine produces population and death surface plots from the gam object b produced by posum()
# plots are on an m by m mesh fdi is the proportion of the mesh spacing to use as a finite difference
# interval for death rate calculations. 
{ data<-b$data # the original data
  a0<-min(data$a0);
  a1<-max(b$amax.fn(data$t))
  t0<-min(data$t);t1<-max(data$t)
  da<-(a1-a0)/(m-1);dt<-(t1-t0)/(m-1)
  ts<-as<-0:(m-1);as<-as*da+a0;ts<-ts*dt+t0
  fd<-min(c(da,dt))*fdi
  t<-rep(ts,rep(m,m));a<-rep(as,m)
  d<-data.frame(t=t,a=a,adult=rep(0,m*m),juvenile=rep(1,m*m))
  #xp<-predict.gam(b,d,type="lpmatrix")
  xp<-posum.pop.matrix(b,d)
  #d$t<-t+fd;d$a<-a+fd
  #xd<-predict.gam(b,d,type="lpmatrix")
  #xd<-(xp-xd)/fd
  xd<-posum.death.matrix(b,d,fd)
  xx<-xp%*%b$coeff;rm(xp)
  xx[a>b$amax.fn(t)]<-NA # hide off surface stuff
  zp<-matrix(xx,m,m)
  xx<-xd%*%b$coeff;rm(xd)
  xx[a>b$amax.fn(t)]<-NA # hide off surface stuff
  zd<-matrix(xx,m,m)
  #print(range(zd));print(range(zp))
  #par(mfrow=c(1,2))
  if (con)
  { contour(as,ts,zp)
    contour(as,ts,zd)
  } else
  { persp(as,ts,zp,theta=theta,xlab="age",ylab="time",zlab="population",col="lightblue",
          zlim=c(0,max(zp,na.rm=TRUE)))
    persp(as,ts,zd,theta=theta,xlab="age",ylab="time",zlab="death rate",col="lightblue",
          zlim=c(0,max(zd,na.rm=TRUE)))
  }
}

posum.stage.plot<-function(b,fit=TRUE)
# produce stage specific plots from an object returned from posum(), if 
# fit=FALSE then only the raw data is plotted
{ d<-b$data
  st<-unique(d$stage) # array of stage numbers
  ns<-length(st)
  for (k in 1:ns)
  { y<-d$y[d$stage==st[k]];t<-d$t[d$stage==st[k]]
    plot(t,y,xlab="time",ylab="population",main=paste("stage",k))
    if (fit)
    { f<-fitted(b)[d$stage==st[k]]
      lines(t,f)
    }
  }
}

posum.fit<-function (G, start = NULL, etastart = NULL, mustart = NULL, family = gaussian(), 
    control = gam.control(), nb.iter = NULL) 
{
    conv <- FALSE
    nobs <- NROW(G$y)
    nvars <- NCOL(G$X)
    y <- G$y
    X <- G$X
    mu.floor<-min(y[y!=0])*1e-5 # NOTE !!! different from gam.fit
    if (nvars == 0) 
        stop("Model seems to contain no terms")
    if (family$family == "gaussian" && family$link == "identity") 
        olm <- TRUE
    else olm <- FALSE
    if (G$m > 0) {
        k1 <- 0
        S.size <- array(0, G$m)
        for (i in 1:G$m) {
            k0 <- k1
            k1 <- k0 + G$df[i]^2
            k0 <- k0 + 1
            S.size[i] <- mean(abs(G$S[k0:k1]))
        }
    }
    weights <- G$w
    offset <- G$offset
    variance <- family$variance
    dev.resids <- family$dev.resids
    aic <- family$aic
    linkinv <- family$linkinv
    mu.eta <- family$mu.eta
    if (!is.function(variance) || !is.function(linkinv)) 
        stop("illegal `family' argument")
    valideta <- family$valideta
    if (is.null(valideta)) 
        valideta <- function(eta) TRUE
    validmu <- family$validmu
    if (is.null(validmu)) 
        validmu <- function(mu) TRUE
    if (is.null(mustart)) {
        eval(family$initialize)
    }
    else {
        mukeep <- mustart
        eval(family$initialize)
        mustart <- mukeep
    }
    if (NCOL(y) > 1) 
        stop("y must be univariate unless binomial")
    coefold <- NULL
    eta <- if (!is.null(etastart)) 
        etastart
    else if (!is.null(start)) 
        if (length(start) != nvars) 
            stop(paste("Length of start should equal", nvars, 
                "and correspond to initial coefs."))
        else {
            coefold <- start
            offset + as.vector(if (NCOL(G$X) == 1) 
                G$X * start
            else G$X %*% start)
        }
    else family$linkfun(mustart)
    mu <- linkinv(eta)
    if (!(validmu(mu) && valideta(eta))) 
        stop("Can't find valid starting values: please specify some")
    devold <- sum(dev.resids(y, mu, weights))
    boundary <- FALSE
    scale <- G$sig2
    pcp<-G$p       ## NOTE!!!! DIFFERENT FROM gam.fit()
    for (iter in 1:(control$maxit + control$globit)) {
        good <- weights > 0
        varmu <- variance(mu)[good]
        if (any(is.na(varmu))) 
            stop("NAs in V(mu)")
        if (any(varmu == 0)) 
            stop("0s in V(mu)")
        mu.eta.val <- mu.eta(eta)
        if (any(is.na(mu.eta.val[good]))) 
            stop("NAs in d(mu)/d(eta)")
        good <- (weights > 0) & (mu.eta.val != 0)
        if (all(!good)) {
            conv <- FALSE
            warning(paste("No observations informative at iteration", 
                iter))
            break
        }
        z <- G$y <- (eta - offset)[good] + (y - mu)[good]/mu.eta.val[good]
        w <- G$w <- sqrt((weights[good] * mu.eta.val[good]^2)/variance(mu)[good])
        G$w <- G$w^2
        G$X <- X[good, ]
        if (dim(X)[2] == 1) 
            dim(G$X) <- c(length(X[good, ]), 1)
        ngoodobs <- as.integer(nobs - sum(!good))
        ncols <- as.integer(1)
        G$sig2 <- scale
        if (G$m > 0 && sum(!G$fix > 0)) {
            temp.sp <- G$sp[!G$fix]
            temp.S.size <- S.size[!G$fix] * temp.sp
            if (min(temp.sp) > 0 && min(temp.S.size) < Machine()$double.eps^0.5 * 
                max(temp.S.size)) 
                G$sp[!G$fix] <- -1
            if (control$trace) 
                cat("Re-initializing smoothing parameters\n")
        }
        if (iter > control$globit) {
            G$target.edf <- G$nsdf + sum(G$edf)
        }
        else G$target.edf <- -1
        G <- mgcv(G)
        G$p<-pcp   ## NOTE!!!! DIFFERENT FROM gam.fit()
        G$p<-pcls(G)  ## NOTE!!!! DIFFERENT FROM gam.fit() 
        if (control$trace) {
            cat("sp: ", G$sp, "\n")
            plot(G$conv$edf, G$conv$score, xlab = "EDF", ylab = "GCV/UBRE score", 
                type = "l")
            points(G$nsdf + sum(G$edf), G$gcv.ubre, pch = 20, 
                col = 2)
        }
        if (any(!is.finite(G$p))) {
            conv <- FALSE
            warning(paste("Non-finite coefficients at iteration", 
                iter))
            break
        }
        start <- G$p
        eta <- drop(X %*% start)
        eta<-pmax(mu.floor,eta) ## NOTE!!!! DIFFERENT FROM gam.fit()    
        mu <- linkinv(eta <- eta + offset)
        dev <- sum(dev.resids(y, mu, weights))
        if (control$trace) 
            cat("Deviance =", dev, "Iterations -", iter, "\n")
        boundary <- FALSE
        if (!is.finite(dev)) {
            if (is.null(coefold)) 
                stop("no valid set of coefficients has been found:please supply starting values", 
                  call. = FALSE)
            warning("Step size truncated due to divergence", 
                call. = FALSE)
            ii <- 1
            while (!is.finite(dev)) {
                if (ii > control$maxit) 
                  stop("inner loop 1; can't correct step size")
                ii <- ii + 1
                start <- (start + coefold)/2
                eta <- drop(X %*% start)
                mu <- linkinv(eta <- eta + offset)
                dev <- sum(dev.resids(y, mu, weights))
            }
            boundary <- TRUE
            if (control$trace) 
                cat("New Deviance =", dev, "\n")
        }
        if (!(valideta(eta) && validmu(mu))) {
            warning("Step size truncated: out of bounds.", call. = FALSE)
            ii <- 1
            while (!(valideta(eta) && validmu(mu))) {
                if (ii > control$maxit) 
                  stop("inner loop 2; can't correct step size")
                ii <- ii + 1
                start <- (start + coefold)/2
                eta <- drop(X %*% start)
                mu <- linkinv(eta <- eta + offset)
            }
            boundary <- TRUE
            dev <- sum(dev.resids(y, mu, weights))
            if (control$trace) 
                cat("New Deviance =", dev, "\n")
        }
        if (abs(dev - devold)/(0.1 + abs(dev)) < control$epsilon || 
            olm) {
            conv <- TRUE
            coef <- start
            break
        }
        else {
            devold <- dev
            coefold <- coef <- start
        }
    }
    if (!conv) {
        if (is.null(nb.iter)) 
            warning("Algorithm did not converge")
        else warning("gam.fit didn't converge at nb iteration ", 
            nb.iter)
    }
    if (boundary) 
        warning("Algorithm stopped at boundary value")
    eps <- 10 * .Machine$double.eps
    if (family$family == "binomial") {
        if (any(mu > 1 - eps) || any(mu < eps)) 
            warning("fitted probabilities numerically 0 or 1 occurred")
    }
    if (family$family == "poisson") {
        if (any(mu < eps)) 
            warning("fitted rates numerically 0 occurred")
    }
    residuals <- rep(NA, nobs)
    residuals[good] <- z - (eta - offset)[good]
    nr <- min(sum(good), nvars)
    wt <- rep(0, nobs)
    wt[good] <- w^2
    wtdmu <- sum(weights * y)/sum(weights)
    nulldev <- sum(dev.resids(y, wtdmu, weights))
    n.ok <- nobs - sum(weights == 0)
    nulldf <- n.ok
    list(coefficients = as.vector(coef), residuals = residuals, 
        fitted.values = mu, family = family, linear.predictor = eta, 
        deviance = dev, null.deviance = nulldev, iter = iter, 
        weights = wt, prior.weights = weights, df.null = nulldf, 
        y = y, converged = conv, sig2 = G$sig2, edf = G$edf, 
        boundary = boundary, sp = G$sp, df = G$df, nsdf = G$nsdf, 
        Vp = G$Vp, mgcv.conv = G$conv, gcv.ubre = G$gcv.ubre)
}

.First.lib <- function(lib, pkg) {
    library(mgcv)
    cat("This is Posum 2.0\n")
}

