\name{simulated surfaces}
\alias{death.surface}
\alias{population.surface}
\title{Simulated total population/death-rate surface (posum)}
\description{ Produces and plots the total death rate surface or population 
surface corresponding to the model specified by \code{\link{pcdr}} and 
code{\link{rec}}.}
\usage{ death.surface(a1,t1,m,con=FALSE)

population.surface(a1,t1,m,con=FALSE)}
\arguments{
\item{a1}{surfaces run from age 0 to age \code{a1}.}
\item{t1}{surfaces run from time 0 to time \code{t1}.}
\item{m}{surfaces will be on an m by m grid.}
\item{con}{\code{TRUE} for a contour plot, \code{FALSE} for a perspective 
plot.}
}

\details{ Given the model specified in functions \code{rec} and \code{pcdr}, 
these functions make use of functions \code{population} and \code{survival}
to produce total death rate (i.e. numbers per unit age interval dying per 
unit time) and population (numbers per unit age interval) surfaces.
}
\value{ A three item list the elements of which are suitable for plotting 
with \code{persp} or \code{contour}:
\item{a}{array of ages.}
\item{t}{array of times.}
\item{pop/death}{matrix of populations or total deaths at all \code{a}, 
code{t} combinations.}
}
\references{
\url{http://www.ruwpa.st-and.ac.uk/simon.html}
}
\author{ Simon N. Wood \email{snw@st-and.ac.uk}}

\section{WARNING }{} 

\seealso{   
\code{\link{pcdr}},
\code{\link{population}},
\code{\link{rec}},
\code{\link{survival}},
}

\examples{
op<-par(mfrow=c(2,2))
pop<-population.surface(30,100,30)
death<-death.surface(30,100,30)
# corresponding per capita death rate ....
pcdr<-death
pcdr$death<-pcdr$death/pop$pop
persp(pcdr$a,pcdr$t,pcdr$death,theta=60)
population.surface(30,100,30,con=TRUE)
par(op)
}
\keyword{models} \keyword{smooth} \keyword{regression}%-- one or more ..




