\name{hyman.filter}
\alias{hyman.filter}
\title{Co-monotonic filtering of cubic Hermite polynomials}
\description{ Filters the coefficients of an interpolating set of cubic 
Hermite polynomials to ensure co-monotonicity of the interpolant. Usually the 
original coefficients will be chosen so that the original interpolant is a 
cubic spline. Hyman's algorithm works with a set of sufficient conditions for
monotonicity so these are slightly more restrictive than is absolutely 
necessary.
}
\usage{hyman.filter(z)
}
\arguments{
\item{z}{ A list or dataframe containing at least the following:
\item{x,y}{set of x,y points that are to be interpolated - interpolant will 
be a function of x.}
\item{b}{gradients of the interpolant w.r.t. x at the x values. It is these 
values that will be modified to ensure co-monotonicity. }
}
If \code{z} has other elements, they will be ignored.
}

\details{ Basically limits on allowable values of \code{b} can be obtained 
from looking at the gradients of the piecewise linear interpolant through 
\code{x,y}: see Hyman (1983) for full details.
}
\value{ Returns the input dataframe/list \code{z} with array \code{b} modified 
to ensure co-monotonicity of the cubic Hermite interpolant defined by 
\code{x,y,b}.
}
\references{
Hyman (1983) SIAM J. Sci. Stat. Comput. 4(4):645-654
}
\author{ Simon N. Wood \email{snw@st-and.ac.uk}}

\section{WARNING }{} 

\seealso{   
\code{\link{cm.splinefun}},\code{\link{spl.coef.conv}}
}

\examples{
# example partly modified from cm.splinefun
x<-0:9
y<-c(0,0,2,2,4,4,5,6,7,7)
normal.spl<-splinefun(x,y)
xx<-seq(0,9,length=100)
plot(xx,normal.spl(xx),type="l",col=2,main="Hyman filter demonstration")
n<-length(y)
z <- .C("spline_coef", method = as.integer(2), n = n, 
        x = as.double(x), y = as.double(y), b = double(n), c = double(n), 
        d = double(n),e = double(0), PACKAGE = "base")
z<-hyman.filter(z) # filter gradients for co-monotonicity
z<-spl.coef.conv(z) # force other coefficients to consistency
cm.spl<-function(x) { .C("spline_eval", z$method, length(x), x = as.double(x), 
            y = double(length(x)), z$n, z$x, z$y, z$b, z$c, z$d, 
            PACKAGE = "base")$y}
lines(xx,cm.spl(xx),col=3)
points(x,y)
}
\keyword{models} \keyword{smooth} \keyword{regression}%-- one or more ..









