\name{Simulate posum data}
\alias{population.data}
\alias{sim.age.bound}
%- Also NEED an `\alias' for EACH other topic documented here.
\title{Population Surface Data Simulation}
\description{ \code{population.data} simulates stage structured population data from the 
model specified by \code{rec} and \code{pcdr}. Stages are really age classes, with the (lower) age 
boundaries specified in \code{sim.age.bound}.}

\usage{
population.data(t=(0:20)*5,ns=11,sc=1,fam="n",adult=FALSE)

sim.age.bound(stage,t)
}
%- maybe also `usage' for other objects documented here.
\arguments{
\item{t}{ The array of sample times at which simulated data are required (\code{population.data}) 
or the array of times at which stage age boundaries are required (\code{sim.age.bound}).} 
\item{ns}{The number of stages.}
\item{sc}{The scale parameter for the noise to be added to the data.}
\item{fam}{Specifies the noise family as one of \code{"n"}, \code{"p"}, or \code{"g"}, 
for normal, Poisson or gamma distributions.}
\item{adult}{\code{TRUE} to simulate an adult final stage (i.e. indeterminate duration), 
\code{FALSE} otherwise. }
\item{stage}{which stage the lower age boundary is required for.}
}

\details{ Stage populations are obtained by integrating the populaiton surface model given by
\code{population} (and based on \code{pcdr} and \code{rec}) between the age limits specified in 
\code{sim.age.bound}. Noise is added according to the specified distribution and scale parameter.
Integration is performed numerically using the trapezoidal rule.}

\value{ \code{sim.age.bound} returns an array of stage lower age boundaries.

\code{population.data} returns a list object suitable for input to \code{posum}. The items in 
this list are:
\item{y}{array of simulated stage populations.}
\item{a0,a1}{lower and upper age boundaries corresponding to the data in \code{y}.}
\item{t}{sample times corresponding to data in \code{y}.}
\item{stage}{stage index corresponding to data in \code{y}.}
}
\references{
\url{http://www.ruwpa.st-and.ac.uk/simon.html}
}
\author{ Simon N. Wood \email{snw@st-and.ac.uk}}

\section{WARNING }{} 

\seealso{   
\code{\link{death.surface}},
\code{\link{pcdr}},
\code{\link{population}},
\code{\link{population.surface}},
\code{\link{rec}},
\code{\link{survival}},
}

\examples{
data<-population.data(fam="p",adult=TRUE) #simulate data
b<-posum(data,fam="p")
op<-par(mfrow=c(2,2))
posum.plot(b)
plot(fitted(b),residuals(b))
plot(data$y,fitted(b))
par(mfrow=c(3,4))
posum.stage.plot(b)
posum.stage(b,lo=1,hi=4)
rm(data,b)
par(op)
}
\keyword{models} \keyword{smooth} \keyword{regression}%-- one or more ..




