\name{posum.X}
\alias{posum.X}
%- Also NEED an `\alias' for EACH other topic documented here.
\title{Obtain model matrix for a posum model}
\description{ Given a posum input dataframe and a \code{gam} object set up to 
provide bases for the components of the posum model, this routine produces 
the posum model matrix: the main part of this is integrating under the 
populaiton surface in order to get the matrix mapping the parameters to
the stage populaiton data. The routine is called by \code{posum}, rather
than directly by the user.}
\usage{
posum.X(data,b,ni=10)
}
%- maybe also `usage' for other objects documented here.
\arguments{
\item{data}{A data frame containing the following named variables:
   \item{y}{The stage population data. This may contain \code{NA}'s: 
            a feature whose prime function is to allow the user to 
            supply extra information about stage boundaries, if
            needed.}
   \item{stage}{Which stage each element of \code{y} relates to.}
   \item{a0}{The lower age boundary for this observation.}
   \item{a1}{The upper age boundary for this observation. \code{NA} 
             implies an adult stage. If an element is equal to the
             corresponding \code{a0} element then the corresponding
             \code{y} is taken as a direct observation of the population
             surface height (rather than the usual integral under it.)}
   \item{t}{Sample time for each observation.}
}
\item{b}{a \code{gam} object containing the information specifying the 
basis for the populaiton surface and optionally the adults.}
\item{ni}{the number of points to use in each (trapezoidal rule) stage 
integration - in practice controlled via option \code{n.int} in 
call to \code{posum.options} as part of \code{posum} call.}
}

\details{ \code{posum} sets up a population surface model as a \code{gam} using the 
functions provided in library \code{mgcv}, however since the response data for the
posum models consist of integrals of the population surface it is necessary to create a 
special \code{gam} design matrix to take account of this. This is done by passing a 
\code{gam} object to \code{posum.X}, which then uses it to call \code{predict.gam} with 
\code{type="lpmatrix"} in order to get the matrix mapping the parameters to the populaiton 
surface at a fine mesh of points appropriately spread over the population surface. 
Summing rows of this matrix in the right way yields the desired model matrix.

}
\value{ The function returns the required model matrix.}

}
\references{
 
\url{http://www.ruwpa.st-and.ac.uk/simon.html}
}
\author{ Simon N. Wood \email{snw@st-and.ac.uk}}

\section{WARNING }{} 

\seealso{   
\code{\link{posum}},
\code{\link{posum.con}},
\code{\link{posum.options}},
}

\keyword{models} \keyword{smooth} \keyword{regression}%-- one or more ..




