\name{posum.con}
\alias{posum.con}
%- Also NEED an `\alias' for EACH other topic documented here.
\title{Population surface model constraint generation}
\description{ Produces the linear inequality constraints that ensure 
(approximate) non-negativeness of model population and death rates. 
This routine is called by \code{posum} during model set up in order 
to generate the required constraints. 
}
\usage{
posum.con(b,amax,data,fdi=0.01,m=40,adults=FALSE)
}
%- maybe also `usage' for other objects documented here.
\arguments{
\item{b}{A \code{gam} object containing the bases used to represent the population 
surface and  adults if present.}
\item{amax}{A function returning the upper age boundary of the populaiton surface 
at any time. Usually this will have been generated by routine \code{age.max}.}
\item{data}{A data frame containing the following named variables:
\item{y}{The stage population data. This may contain \code{NA}'s: 
            a feature whose prime function is to allow the user to 
            supply extra information about stage boundaries, if
            needed.}
\item{stage}{Which stage each element of \code{y} relates to.}
\item{a0}{The lower age boundary for this observation.}
\item{a1}{The upper age boundary for this observation. \code{NA} 
             implies an adult stage. If an element is equal to the
             corresponding \code{a0} element then the corresponding
             \code{y} is taken as a direct observation of the population
             surface height (rather than the usual integral under it.)}
\item{t}{Sample time for each observation.}
}
\item{fdi}{coefficient controlling the size of finite difference interval to use for
numerical calculation of derivatives required in constraints. Constraints are applied
at each point on a regular \code{m} by \code{m} grid over the age time plane. 
The minimum of the grid spacing in age and time is found, and multiplied by 
\code{fdi} to get the finite difference interval to use.}
\item{m}{constraints are applied on an \code{m} by \code{m} grid over the model age 
time plane.}
\item{adults}{set to \code{TRUE} if adult constraints are required, \code{FALSE} 
otherwise.}
}

\details{ Let \eqn{ n }{n} be the population per unit age interval. If 
\eqn{ n \ge 0 }{n>=0} along the upper age boundary of the surface, and final sample 
time boundary of the surface, and if:
\eqn{ \partial n/\partial a+\partial n/\partial t \le 0 }{dn/da+dn/dt <=0} everywhere,
 then n will be positive and will not imply negative death rates. These constraints 
are applied in practice on a relatively fine mesh across the age-time domain of 
\eqn{ n }{n}. 

If there is an adult stage \eqn{ a }{a} then additional constraints are required 
(partly in replacement for the surface non-negativity constraint at the upper age 
boundary). The adult stage is governed by:
\eqn{ da/dt = R(t) - m(t) a(t) }{da/dt=R(t)-m(t)a(t)}
where \eqn{ m }{m} is per capita death rate and \eqn{ R(t) }{R(t)} is recruitment to the 
adult stage. 
\eqn{ R(t)= n(T)[1-\partial T/\partial t] }{R(t)=n(T)[1-dT/dt]} where \eqn{ T }{T} is 
the age at the upper boundary of the surface. We require that \eqn{ a \ge 0 }{a>=0} and 
\eqn{ R(t)-da/dt\ge 0 }{R(t)-da/dt >=0} to ensure non-negativeness of the adult 
population and its death rate respectively. Once again, these constraints are 
applied at on a fine time mesh in practice. 

The routine is called by \code{posum}, rather than being used directly: 
\code{fdi} and \code{m} would usually be set using \code{posum.options} in a 
call to \code{posum}. 
}
\value{ A matrix \code{C} is returned such that if 
\code{C b$coeff>=0} 
then the 
(approximate) constraints are met. 
}
\references{
\url{http://www.ruwpa.st-and.ac.uk/simon.html}
}
\author{ Simon N. Wood \email{snw@st-and.ac.uk}}

\section{WARNING }{} 

\seealso{   
\code{\link{posum}},
\code{\link{posum.options}}
}


\keyword{models} \keyword{smooth} \keyword{regression}%-- one or more ..




