\name{posum.pop.matrix}
\alias{posum.pop.matrix}
\alias{posum.death.matrix}
%- Also NEED an `\alias' for EACH other topic documented here.
\title{Population/Death surface evaluation matrices}
\description{ These two routines find the matrices mapping the model 
parameter vector to either the population surface or the total death 
rate surface. \code{posum.pop.matrix} also deals with adults. The routines are 
primarily designed as internal service routines. }
\usage{ posum.pop.matrix(b,d,t0=0,point.wise=TRUE)

posum.death.matrix(b,d,fd)
}
%- maybe also `usage' for other objects documented here.
\arguments{
\item{b}{ A fitted \code{gam} object for a posum model with at least the extra 
variables \code{target.dt} and \code{model.type}.}
\item{d}{
A data frame defining the ages/times at which evaluation is required. It should 
contain variables:
   \item{a}{ array of ages at which evaluation is required.}
   \item{t}{ array of times at which evaluation required.}
   \item{juvenile}{by variable for juveniles.}
   \item{adult}{by variable for adults. }

The latter two are not always required.
}
\item{t0}{ The initial time (\code{posum.pop.matrix} only).}
\item{point.wise}{ If this is true then the surfaces at each age, time point are 
evaluated with spearate calls to \code{predict.gam}, which saves memory. If false 
then there is just one call to \code{predict.gam} with one large data frame: this 
is faster, but uses a large amount of memory. (\code{posum.pop.matrix} only).}
\item{fd}{ finite difference interval to use for obtaining death rates from 
type 1 models.}
}

\details{The required matrices are obtained by calls to \code{predict.gam()} 
which obtain matrices mapping parameters to evaluations of the model at particular 
age, time points. Appropriate quadrature formulae are then applied to the matrix 
rows. 
}
\value{ The functions return the required matrices.
}
\references{

Gu and Wahba (1991) Minimizing GCV/GML scores with multiple smoothing parameters 
via the Newton method. SIAM J. Sci. Statist. Comput. 12:383-398


Wood (2000) Modelling and Smoothing Parameter Estimation 
with Multiple Quadratic Penalties. JRSSB 62(2):413-428

Wood (2003) Thin Plate Regression Splines in press JRSSB
 
\url{http://www.ruwpa.st-and.ac.uk/simon.html}
}
\author{ Simon N. Wood \email{snw@st-and.ac.uk}}

\section{WARNING }{} 

\seealso{   
\code{\link{posum}}
\code{\link{age.max}},
\code{\link{cm.splinefun}},
\code{\link{death.surface}},
\code{\link{hyman.filter}},
\code{\link{pcdr}},
\code{\link{population}},
\code{\link{population.data}},
\code{\link{population.surface}},
\code{\link{posum.atplot}},
\code{\link{posum.check}},
\code{\link{posum.con}},
\code{\link{posum.fit}},
\code{\link{posum.options}},
\code{\link{posum.plot}},
\code{\link{posum.stage}},
\code{\link{posum.stage.plot}},
\code{\link{posum.X}},
\code{\link{rec}},
\code{\link{sim.age.bound}},
\code{\link{spl.coef.conv}},
\code{\link{stage.bound}},
\code{\link{survival}},
}

\keyword{models} \keyword{smooth} \keyword{regression}%-- one or more ..






