\name{posum.stage}
\alias{posum.stage}
%- Also NEED an `\alias' for EACH other topic documented here.
\title{Extract stage specific information from a fitted posum model}
\description{Given a fitted model object as returned by \code{posum}, this function integrates the 
fitted population and death rate surfaces over stages \code{lo} to \code{hi} inclusive, to produce 
rates averaged over stage(s) at each of the points in array \code{t}. If \code{aggregate} is \code{TRUE},
then integrated quantities over all elements of \code{t} are produced.  }
\usage{
posum.stage(x,lo=1,hi=lo,t=seq(min(x$data$t),max(x$data$t),length=100),
                      n.int=x$options$n.int,fdi=x$options$fdi,plot=TRUE,aggregate=FALSE)
}
%- maybe also `usage' for other objects documented here.
\arguments{
\item{x}{A fitted model object as produced by \code{posum}.}
\item{lo}{The stage at which to start integration}
\item{hi}{The stage at which to stop integration (if equal to \code{lo} then only one stage is 
integrated over).}
\item{t}{array of times at which to evaluate populations and death rates. If \code{aggregate==TRUE}
then this is an integration mesh.}
\item{n.int}{The number of age points to use in the integration over stages.}
\item{fdi}{The parameter controlling the finite difference interval used to evaluate death rates. The 
average spacing used in the integration mesh is found and then multiplied by this number to get the 
finite difference interval used for gradient estimation.}
\item{plot}{if \code{TRUE} then a plot is produced. }
\item{aggregate}{if \code{TRUE} then \code{t} is treated as an integration mesh, and rather than producing 
estimates for each time in \code{t}, integrated estimates are produced.}
}

\details{ All numerical inegration is by the trapezoidal rule. All evaluations use \code{predict.gam}. 
Standard errors are produced for all quantities, including per capita death rate (the latter se by 
local linearization).
}
\value{ A dataframe is returned with the following elements:
\item{p}{estimated population(s).}
\item{d}{estimated total death rate(s).}
\item{m}{estimated per capita death rate(s).}
\item{p.se}{standard error for \code{p}.}
\item{d.se}{standard error for \code{d}.}
\item{m.se}{standard error for \code{m}.}

}
\references{
\url{http://www.ruwpa.st-and.ac.uk/simon.html}
}
\author{ Simon N. Wood \email{snw@st-and.ac.uk}}

\section{WARNING }{} 

\seealso{   
\code{\link{posum.atplot}},
\code{\link{posum.plot}},
\code{\link{posum.stage.plot}},
}

\examples{
data<-population.data(fam="p",adult=TRUE) #simulate data
b<-posum(data,fam="p")
r<-posum.stage(b,1,10,aggregate=TRUE)
old.pcdr<-pcdr;old.rec<-rec;old.sim.age.bound<-sim.age.bound
rec<-function(t) {50*exp(-0.5*(t-5)^2)}
pcdr<-function(a,t) {0.2231+a*0}
sim.age.bound<-function(stage,t) { (stage-1)+t*0}
data<-population.data(t=0:9,ns=9,sc=0,fam="n",adult=FALSE) #simulate data
b<-posum(data,fam="n")
r<-posum.stage(b,1,9,aggregate=TRUE)
pcdr<-old.pcdr;rec<-old.rec;sim.age.bound<-old.sim.age.bound
}
\keyword{models} \keyword{smooth} \keyword{regression}%-- one or more ..




