\name{stage.bound}
\alias{stage.bound}
\title{Extract stage boundary information from posum objects}
\description{Takes and input or output posum data object and extracts
the stage boundary information from this, returning a function giving
the boundary as a function of time, which interpolates the extracted 
boundary data and meets the biological constraint that its time derivative 
is always less than or equal to 1.}

\usage{
stage.bound(x,stage,low=TRUE)
}
%- maybe also `usage' for other objects documented here.
\arguments{
\item{x}{A data frame containing the following named variables:
   \item{y}{The stage population data. This may contain \code{NA}'s: 
            a feature whose prime function is to allow the user to 
            supply extra information about stage boundaries, if
            needed.}
   \item{stage}{Which stage each element of \code{y} relates to.}
   \item{a0}{The lower age boundary for this observation.}
   \item{a1}{The upper age boundary for this observation. \code{NA} 
             implies an adult stage. If an element is equal to the
             corresponding \code{a0} element then the corresponding
             \code{y} is taken as a direct observation of the population
             surface height (rather than the usual integral under it.)}
   \item{t}{Sample time for each observation.}
}
 \item{stage}{The stage for which the age boundary function is required.}
\item{low}{\code{TRUE} if a lower boundary is required, \code{FALSE} for an upper boundary.}
}

\details{ All \code{a0} or \code{a1} records are extracted for the relevant 
stage, along with their corresponding \code{t} records. After some checks, 
these data are then interpolated using \code{cm.splinefun}, which fits a 
cubic interpolating spline and then filters its coefficients it to ensure that 
its time derivative is never greater than 1. The resulting filtered 
interpolating function is then returned.
}
\value{ An interpolating function of time.
}
\references{
\url{http://www.ruwpa.st-and.ac.uk/simon.html}
}
\author{ Simon N. Wood \email{snw@st-and.ac.uk}}

\section{WARNING }{} 

\seealso{   
\code{\link{age.max}},
\code{\link{cm.splinefun}},
\code{\link{hyman.filter}},
\code{\link{posum.atplot}},
\code{\link{spl.coef.conv}},
}

\examples{
data<-population.data(fam="p",adult=TRUE) #simulate data
bound3<-stage.bound(data,3,low=TRUE)
time<-0:100
plot(time,bound3(time),type="l",
main="Simulated data stage 3 lower age boundary")
}
\keyword{models} \keyword{smooth} \keyword{regression}%-- one or more ..




