\name{Simulating posum data}
\alias{population}
\alias{survival}

\title{Population Surface Model Simulation}
\description{ These routines are for numerically integrating the McKendrick equation to obtain
population per unit age interval from the per capita death rates and recruitment rates specified
in \code{rec} and \code{pcdr}. \code{survival(a,t)} finds the proportion surviving from birth to age 
a, t, while \code{population(a,t)} finds the populaiton per unit age interval at a, t using \code{rec} 
and \code{survival}.}
\usage{
survival(a,t)

population(a,t)
}
%- maybe also `usage' for other objects documented here.
\arguments{
\item{a}{ age survived to or at which population is required.}
\item{t}{ time survived to or at which population is required.}
}

\details{ Survival is the negative exponential of the integral of per capiat death rate 
from recruitment to a, t. The integral is performed numerically using the trapezoidal rule.
The population per unit age interval at a,t  is just the product of the recruitment rate at 
t-a and the survival to a, t. Note that these functions do not operate on arrays.
}
\value{ Returns the proportion surviving, or the population per unit age interval.
}
\references{
\url{http://www.ruwpa.st-and.ac.uk/simon.html}
}
\author{ Simon N. Wood \email{snw@st-and.ac.uk}}

\section{WARNING }{} 

\seealso{   
\code{\link{death.surface}},
\code{\link{pcdr}},
\code{\link{population.data}},
\code{\link{population.surface}},
\code{\link{rec}},
\code{\link{sim.age.bound}},
}

\examples{
m<-30;a1<-30;t1<-100
pop<-matrix(0,m,m)
da<-a1/(m-1);dt<-t1/(m-1)
t<-a<-0:(m-1);a<-a*da;t<-t*dt
for (i in 1:m) for (j in 1:m) pop[i,j]<-population(a[i],t[j])
contour(a,t,pop,xlab="a",ylab="t") 
}
\keyword{models} \keyword{smooth} \keyword{regression}%-- one or more ..




